@@ -14,7 +14,12 @@
 from os.path import isfile, splitext
 import numpy as np
 from scipy.stats import beta
-from dvhastats.utilities import dict_to_array, csv_to_dict
+from dvhastats.utilities import (
+    dict_to_array,
+    csv_to_dict,
+    close_plot,
+    moving_avg,
+)
 from dvhastats import plot
 from scipy import stats as scipy_stats
 from sklearn import linear_model
@@ -23,7 +28,7 @@ from regressors import stats as regressors_stats
 
 
 class DVHAStats:
-    def __init__(self, data, var_names=None, x_axis=None):
+    def __init__(self, data, var_names=None, x_axis=None, avg_len=5):
         """Class used to calculated various statistics
 
         Parameters
@@ -37,6 +42,10 @@ class DVHAStats:
         x_axis : numpy.array, list, optional
             Specify x_axis for plotting purposes. Default is based on row
             number in data
+        avg_len : int
+            When plotting raw data, a trend line will be plotted using this
+            value as an averaging length. If N < avg_len + 1 will not
+            plot a trend line
         """
         if isinstance(data, np.ndarray):
             self.data = data
@@ -60,6 +69,10 @@ class DVHAStats:
 
         self.box_cox_data = None
 
+        self.plots = []
+
+        self.avg_len = avg_len
+
     def get_data_by_var_name(self, var_name):
         """Get the single variable array based on var_name"""
         index = self.get_index_by_var_name(var_name)
@@ -69,6 +82,10 @@ class DVHAStats:
         """Get the variable index by var_name"""
         if var_name in self.var_names:
             index = self.var_names.index(var_name)
+        elif isinstance(var_name, int) and var_name in range(
+            self.variable_count
+        ):
+            return var_name
         else:
             msg = "%s is not a valid var_name\n%s" % (
                 var_name,
@@ -111,7 +128,9 @@ class DVHAStats:
         """
         norm, p = np.zeros(self.variable_count), np.zeros(self.variable_count)
         for i in range(self.variable_count):
-            norm[i], p[i] = scipy_stats.normaltest(self.data[:, i])
+            norm[i], p[i] = scipy_stats.normaltest(
+                self.data[:, i], nan_policy="omit"
+            )
         return norm, p
 
     def linear_reg(self, y, saved_reg=None):
@@ -258,24 +277,39 @@ class DVHAStats:
             self.data[:, index], alpha=alpha, lmbda=lmbda
         )
 
+        return self.box_cox_data[:, index]
+
     def box_cox(self, alpha=None, lmbda=None):
         """Apply box_cox_by_index for all data"""
         for i in range(self.variable_count):
             self.box_cox_by_index(i, alpha=alpha, lmbda=lmbda)
 
+    def add_tend_line(self, var_name, plot_index):
+        trend_x, trend_y = moving_avg(
+            self.get_data_by_var_name(var_name), self.avg_len
+        )
+        self.plots[plot_index].add_line(
+            trend_y, x=trend_x, line_color="black", line_width=0.75
+        )
+
     def show(self, var_name):
         """Display a plot of var_name with matplotlib"""
-        if isinstance(var_name, int):
-            index = var_name
-            var_name = self.var_names[index]
-        else:
-            index = self.get_index_by_var_name(var_name)
-        plot.Plot(
-            self.data[:, index],
-            x=self.x_axis,
-            xlabel="Observation",
-            ylabel=var_name,
+        index = self.get_index_by_var_name(var_name)
+        var_name = self.var_names[index]
+        self.plots.append(
+            plot.Plot(
+                self.data[:, index],
+                x=self.x_axis,
+                xlabel="Observation",
+                ylabel=var_name,
+            )
         )
+        self.add_tend_line(var_name, -1)
+        return self.plots[-1].figure.number
+
+    def close(self, figure_number):
+        """Close a plot by figure_number"""
+        close_plot(figure_number, self.plots)
 
 
 def get_lin_reg_p_values(X, y, predictions, y_intercept, slope):
@@ -304,12 +338,6 @@ def get_lin_reg_p_values(X, y, predictions, y_intercept, slope):
         and t-values (t)
     """
 
-    if isinstance(y, list):
-        y = np.array(y)
-
-    if isinstance(predictions, list):
-        predictions = np.array(predictions)
-
     newX = np.append(np.ones((len(X), 1)), X, axis=1)
     mse = (sum((y - predictions) ** 2)) / (len(newX) - len(newX[0]))
 
@@ -523,6 +551,8 @@ class ControlChartData:
         # since moving range is calculated based on 2 consecutive points
         self.scalar_d = 1.128
 
+        self.plots = []
+
     def __str__(self):
         msg = [
             "center_line: %0.3f" % self.center_line,
@@ -534,15 +564,19 @@ class ControlChartData:
     def __repr__(self):
         return str(self)
 
+    @property
+    def y_no_nan(self):
+        return self.y[~np.isnan(self.y)]
+
     @property
     def center_line(self):
         """Center line"""
-        return np.mean(self.y)
+        return np.mean(self.y_no_nan)
 
     @property
     def avg_moving_range(self):
         """Avg moving range based on 2 consecutive points"""
-        return np.mean(np.absolute(np.diff(self.y)))
+        return np.mean(np.absolute(np.diff(self.y_no_nan)))
 
     @property
     def sigma(self):
@@ -601,16 +635,23 @@ class ControlChartData:
     def show(self):
         """Display the univariate control chart with matplotlib"""
         lcl, ucl = self.control_limits
-        plot.ControlChart(
-            self.y,
-            self.out_of_control,
-            self.center_line,
-            x=self.x,
-            title=self.plot_title,
-            lcl=lcl,
-            ucl=ucl,
-            ylabel=self.var_name,
+        self.plots.append(
+            plot.ControlChart(
+                self.y,
+                self.out_of_control,
+                self.center_line,
+                x=self.x,
+                title=self.plot_title,
+                lcl=lcl,
+                ucl=ucl,
+                ylabel=self.var_name,
+            )
         )
+        return self.plots[-1].figure.number
+
+    def close(self, figure_number):
+        """Close a plot by figure_number"""
+        close_plot(figure_number, self.plots)
 
 
 class HotellingT2:
@@ -637,6 +678,7 @@ class HotellingT2:
         self.plot_title = (
             "Multivariate Control Chart" if plot_title is None else plot_title
         )
+        self.plots = []
 
     def __str__(self):
         msg = [
@@ -720,11 +762,19 @@ class HotellingT2:
 
     def show(self):
         """Display the multivariate control chart with matplotlib"""
-        plot.ControlChart(
-            self.Q,
-            self.out_of_control,
-            self.center_line,
-            title=self.plot_title,
-            ucl=self.ucl,
-            ylabel="Hottelling T^2",
+        self.plots.append(
+            plot.ControlChart(
+                self.Q,
+                self.out_of_control,
+                self.center_line,
+                title=self.plot_title,
+                ucl=self.ucl,
+                lcl=self.lcl,
+                ylabel="Hottelling T^2",
+            )
         )
+        return self.plots[-1].figure.number
+
+    def close(self, figure_number):
+        """Close a plot by figure_number"""
+        close_plot(figure_number, self.plots)
